//
//  bouclier.java
//
// pour definir les donnees qui seront partagees dans les classes
// qui calculent le mouvement des particules :
// pousse (mouvement) et bbb (champ magnetique).
//
//  Fabrice Mottez
//

import simu.*;
import java.util.Vector;

/**
 * pour definir les donnees qui seront partagees dans les classes
 * qui calculent le mouvement des particules
 */
public class Bouclier {


// initialisation des constantes -------------------------------


      static final double DG=6.2832/360., GD=1./DG,  CKIN=593000.;
      static final double MPSME=1836.0;
      static final double CKINI=1./CKIN;
      static final double CELE=1.601E-19, CELEI=1./CELE, AME=9.109E-31 ;
      static final double CSME=1.7587e+11, CSMEI=1./CSME ;
      static final double CSMP=-CSME/MPSME, CSMPI=1./CSMP ;
      static final double CSMO=CSMP/16, CSMOI=1./CSMO ;
      static final double CSMA=CSMP/2, CSMAI=1./CSMO ;
      static final double CONVX=1000., CONVXI=1./CONVX ;

/* general constants
        DG=6.2832/360. 		DEGREES--> RADIANS
        GD=1./DG
        MISME=1836			MASSE DU PROTON / MASSE DE L'ELECTRON
        CKIN=593000.   		FOR ELECTRON KINETIC ENERGY
        CKINI=1./CKIN
        CELE=1.601E-19 		ELECTRON CHARGE
        CELEI=1./CELE
        AME=9.109e-31  		ELECTRON MASS
        CSME=1.7587e+11  	ELECTRON CHARGE/MASS
        CSMI,CSMO,CSMA		IDEM POUR LE PROTON, O+,He++
        CONVX=6 400 000     EARTH RADIUS IN METRES
*/

// cadrage de l'image

       double TailleKM=1000.,TailleMetres=CONVX*TailleKM;
       String TaillePlus="1000", TailleMoins="-1000" ;
// pour communiquer le cadrage de l'image a SimuPlotLive
       double Xrange0=-TailleKM,Xrange1=TailleKM ;
       double Yrange0=-TailleKM,Yrange1=TailleKM ;
       double Zrange0=-TailleKM,Zrange1=TailleKM ;
// un cadre un peu plus grand pour les test : on s'arrete si ca depasse
// c'est employe dans Projectile, PlotLive et Plot.
       double RangeDeborde=1.2;
       double Xrange0p=-TailleKM*RangeDeborde,Xrange1p=TailleKM*RangeDeborde ;
       double Yrange0p=-TailleKM*RangeDeborde,Yrange1p=TailleKM*RangeDeborde ;
       double Zrange0p=-TailleKM*RangeDeborde,Zrange1p=TailleKM*RangeDeborde ;

// rapport de la charge a la masse de la particule
       double CSM, CSMI;

// time step
       double DT, DTION=0.005,DTELE=0.0001;
       double DTS2;

// electric field frequency omega*Dt
       double APHASE=0.;
       double AOMDT=0.1;
//       electric field in V/m
       double EX0=0.00;
       double EY0=0.0;
       double EZ0=0.0;

//	WRITE(6,*) 'AME=',AME,'   AMU=',AMU,'   AMUSME=',AMUSME
//	WRITE(6,*) 'B0 =',B0
// constants
      int NSTEP=50;
//      double NSTEP2=200;
// Frequency of diagnostics
      double IWRCB=100;
      double ndiag=10;

// magnetic field maximum amplitude
      double BZE0=10.e-9 ;  // 10 nanoTesla (magnetopause Terre)
// dans ce cas, DT=0.0005 est OK, mais par precaution  on a choisi DT=0.0001.

//      double BZE0=1000.e-9 ;  // 1000 nanoTesla (zone aurorale 10000km Terre)
// dans ce cas, il faut DT divise par 100, DT=0.000005
// sinon le mouvement des electrons est incorrect.

      double BZE0S2=BZE0/2.;

// declaration de variables qui dependent du temps
// position et vitesse
      double XE,YE,ZE,VXE,VYE,VZE,V2,VV ;
// position in kilometres
       double XER,YER=0.,ZER=0.;
// champ magnetique
      double BXE,BYE,BZE,B2,B3,B ;
// champ electrique
      double EX=0.,EY=0.,EZ=0.;
// necessaire pour l'algorithme de Boris
      double S0X,S0Y,S0Z,SM;
      double TWX,TWY,TWZ;

// intial values

// energy in eV
      double AK0;
// ALPHA is the pitch angle in degrees
      double ALPHA;
// BETA is the XoY angle in degres
      double BETA;

      double   ALRAD, COSA, SINA ;
      double   BERAD ,COSB, SINB ;



/**
 * methode constructeur
 */
public Bouclier(){

}


/**
 * methode pour le calcul du Champ Magnetique
 */
//////////////// CHAMP /////////////////////////////////

// calcul du champ magnetique
public String Champ() throws SimuException {
	String test_Champ;
	      BXE=0.;
	      BYE=0.;
	      BZE0S2=BZE0/2.;
	      double arg=0.01*XE/CONVX; // gradient typiquement sur 100 km.
	      double tanh=(Math.exp(arg)-Math.exp(-arg))/(Math.exp(arg)+Math.exp(-arg));
	      BZE=BZE0S2*(1 +tanh);
// au cas ou on ne trouve pas la tangente hyperbolique
//          BZE=BZE0;

	      B2=BZE*BZE;
	      B=BZE;
      	  B3=BZE*B2;
      	  test_Champ="Champ calcul";

	return(test_Champ);
}

/**
 * methode : initialise les parametres du calcul : vitesses, position, etc
 *
 */
//////////////// INITIALISE /////////////////////////////////

public String Initialise() throws SimuException {

//     charge sur masse, et l'inverse
       CSMI=1./CSM ;

// angles

      ALRAD=ALPHA*DG ;
      COSA=Math.cos(ALRAD) ;
      SINA=Math.sin(ALRAD) ;

      BERAD=BETA*DG ;
      COSB=Math.cos(BERAD) ;
      SINB=Math.sin(BERAD) ;

// velocity in metres/second
       VV=CKIN*Math.sqrt(Math.abs(AK0*CSM/CSME));
       VXE=VV*SINA*COSB;
       VYE=VV*SINA*SINB;
       VZE=VV*COSA;
       V2=VXE*VXE+VYE*VYE+VZE*VZE;

//       DT=Magnetique.dt;
       DTS2=DT/2.;



// position in metres

       XE=CONVX*XER;
       YE=CONVX*YER;
       ZE=CONVX*ZER;

      String test;
      test="initialisation effectuee";
	  return(test);

}


/**
 * methode : on incremente le pas de temps
 * c'est a dire que connaissant la position et la vitesse de
 * la particule a l'instant T, on calcule tout cela a T+DT.
 *
 */
//////////////// INCREMENT /////////////////////////////////

public String Increment() throws SimuException {
	String test;
//	        XE= Projectile.x;
//	        YE= Projectile.y;
//	        VXE=Projectile.vx;
//	        VYE=Projectile.vy;
	        test=Champ();

/*	        System.out.println("-Bouclier- test : "+test);
	        System.out.println("-Bouclier- BXE    ="+BXE);
	        System.out.println("-Bouclier- BZE    ="+BZE);
*/


//   NORMALISATION AVEC E/M
			      BXE=BXE*CSM;
			      BYE=BYE*CSM;
			      BZE=BZE*CSM;
			      EX=EX*CSM;
			      EY=EY*CSM;
			      EZ=EZ*CSM;

//	 carre de la vitesse
			      V2=VXE*VXE+VYE*VYE+VZE*VZE;



//     CALCUL DE V.


// gyrofrequence*DT/2

			      BXE=BXE*DTS2;
			      BYE=BYE*DTS2;
			      BZE=BZE*DTS2;

// rotation de la vitesse par algoritme de Boris

			      S0X=BXE*BXE;
			      S0Y=BYE*BYE;
			      S0Z=BZE*BZE;

			      SM=2.0/(1.0+S0X+S0Y+S0Z);

			      S0X=SM*BXE;
			      S0Y=SM*BYE;
			      S0Z=SM*BZE;

			      TWX   = VXE+VYE*BZE-VZE*BYE;
			      TWY   = VYE+VZE*BXE-VXE*BZE;
			      TWZ   = VZE+VXE*BYE-VYE*BXE;

			      VXE = VXE+  TWY*S0Z -  TWZ*S0Y;
			      VYE = VYE+  TWZ*S0X -  TWX*S0Z;
			      VZE = VZE+  TWX*S0Y -  TWY*S0X;

//    CALCUL DE X AU TEMPS N+1

			      XE=XE+DT*VXE;
			      YE=YE+DT*VYE;
			      ZE=ZE+DT*VZE;

// Position en kilometres

       XER=CONVXI*XE;
       YER=CONVXI*YE;
       ZER=CONVXI*ZE;


// DENORMALISATION PAR RAPPORT A E/M
			      BXE=BXE*CSMI;
			      BYE=BYE*CSMI;
			      BZE=BZE*CSMI;
			      EX=EX*CSMI;
			      EY=EY*CSMI;
			      EZ=EZ*CSMI;


            test="increment effectue";
/*
            System.out.println("alors, toujours en train de merder avec java ?");
            System.out.println("-Bouclier- test : "+test);
            System.out.println("-Bouclier- XE =  "+ XER+",  YE = "+ YER);
            System.out.println("-Bouclier- XE =  "+ XER+",  ZE = "+ ZER);
            System.out.println("-Bouclier- VXE=  "+VXE+",  VYE= "+VYE);
            System.out.println("-Bouclier- VXE=  "+VXE+",  VZE= "+VZE);
*/
        return(test);
    }

}