//
//  Magnetique.java
//  SimuApplet
//
//  Copyright (c) 2001 Observatoire de Paris-Meudon. All rights reserved.
//

import simu.*;
import java.util.Vector;

public class Magnetique extends SimuApplet {

    static double dt = 0.0001;
    static double x,y,z;
    static double xer0,yer0,zer0;
    static double vx,vy,vz;
    static int istep=0;
    String test_increment, test_initialise ;

    Bouclier boubou ;

    public Magnetique() {

		boubou= new Bouclier();

        // paramtres d'entre
        ParamIn particule0 = new ParamIn();
        particule0.type = "string";
        particule0.label = "typeDeParticule";
        particule0.titre = "type de particule";
//        particule0.unit = " ";
        particule0.acquisition = "choix";
        String[] choixParticule = {"electron","ion H+", "ion He++", "ion O+"};
        particule0.choix = choixParticule;
        particule0.defaut = "ion H+";
        addParamIn(particule0);

        ParamIn energie0 = new ParamIn();
        energie0.type = "nombre";
        energie0.label = "energie0";
        energie0.titre = "energie initiale";
        energie0.unit = "electron Volts";
        energie0.acquisition = "choix";
        String[] choixEnergie0 = {"1", "10", "100", "1000","10000","50000" };
        energie0.choix = choixEnergie0;
        energie0.defaut = "100";
        addParamIn(energie0);

        ParamIn alpha0 = new ParamIn();
        alpha0.type = "nombre";
        alpha0.label = "alpha";
        alpha0.titre = "angle d'attaque";
        alpha0.unit = "degres";
        alpha0.acquisition = "choix";
        String[] choixAlpha = {"0","1","5","10", "30", "45", "60","80","89","90" };
        alpha0.choix = choixAlpha;
        alpha0.defaut = "45";
        addParamIn(alpha0);

        ParamIn beta0 = new ParamIn();
        beta0.type = "nombre";
        beta0.label = "beta";
        beta0.titre = "angle d'arrivee";
        beta0.unit = "degres";
        beta0.acquisition = "choix";
        String[] choixBeta = {"-60","-30","0", "30", "45", "60","80","90"};
        beta0.choix = choixBeta;
        beta0.defaut = "30";
        addParamIn(beta0);

       ParamIn xer0 = new ParamIn();
        xer0.type = "nombre";
        xer0.label = "XInitial";
        xer0.titre = "X initial (axe horizontal)";
        xer0.unit = "kilometres";
        xer0.acquisition = "choix";
        String[] choixXer =
        {"-800","-600","-300","-100","-50","0",
         "50","100","200","300","600","800"};
        xer0.choix = choixXer;
        xer0.defaut = "200";
        addParamIn(xer0);


        // paramtres de sortie
        ParamOut xv = new ParamOut();
        xv.type = "nombre";
        xv.label = "x";
        addParamOut(xv);
        ParamOut yv = new ParamOut();
        yv.type = "nombre";
        yv.label = "y";
        addParamOut(yv);
        ParamOut zv = new ParamOut();
        zv.type = "nombre";
        zv.label = "z";
        addParamOut(zv);

        // options d'affichage

// ici un affichage en x-y
// noter l'option zooming, qui semble permettre que l'image de fond se
// recadre quand on clique sur le bouton "ajuster".
        Affichage pyx = new Affichage();
        pyx.type = "plotlive";
        pyx.titre = "mouvement de l'lectron y/x live";
        String[][] pparamsXY = {{"x", "y"}};
        pyx.params = pparamsXY;
        pyx.fond = "bouclier.gif";
        pyx.zooming = true;
        pyx.xdebut 	= boubou.TailleMoins;
        pyx.xfin 	= boubou.TaillePlus;
        pyx.ydebut 	= boubou.TailleMoins;
        pyx.yfin 	= boubou.TaillePlus;
        addAffichage(pyx);


// ici un affichage en x-z
        Affichage pxz = new Affichage();
        pxz.type = "plotlive";
        pxz.titre = "mouvement de l'lectron z/x live";
        String[][] pparamsXZ = {{"x", "z"}};
        pxz.params = pparamsXZ;
        pxz.fond = "bouclier.gif";
        pxz.zooming = true;
        pxz.xdebut 	= boubou.TailleMoins;
        pxz.xfin 	= boubou.TaillePlus;
        pxz.ydebut 	= boubou.TailleMoins;
        pxz.yfin 	= boubou.TaillePlus;
        addAffichage(pxz);

//ici, deux figures affichees en meme temps
        Affichage multi = new Affichage();
        multi.type = "multiplot";
        multi.titre = "mouvement de l'lectron y/x et x/z live";
        multi.sousAff = new Affichage[2];
        multi.sousAff[0] = pyx;
        multi.sousAff[1] = pxz;
        addAffichage(multi);

/*        p = new Affichage();
        p.type = "plot";
        p.titre = "mouvement de l'lectron y/x";
        p.params = pparamsXY;
        p.fond = "apple.gif";
        affichage.addElement(p);
*/

/*        Affichage p = new Affichage();
        p.type = "tableau";
        p.titre = "mouvement de l'lectron";
        addAffichage(p);*/
    appletDoc("magnetiquedoc.xml");
  }

 // ///////////////// INIT CALCUL LIVE /////////////////////////////////////

 // calcul live (utilis avec affichage d'un plot live)
    public void initCalculLive(ListeValeurs in) throws SimuException {

        x = 0;
        y = 0;
        z = 0;
//        xer0=0;
        yer0=0;
        zer0=0;

        String particule0=in.lireString("typeDeParticule");
        System.out.println("particule0 : "+ particule0);

        double energie0 = in.lireDouble("energie0");
	double alpha0   = in.lireDouble("alpha");
        double beta0    = in.lireDouble("beta");
        double xer0     = in.lireDouble("XInitial");



//        boubou= new Bouclier(dt);

        boubou.CSM=boubou.CSME ;

      if (particule0=="electron"){
		  boubou.CSM=boubou.CSME ;
		  boubou.DT=boubou.DTELE ;
		  }
      if (particule0=="ion H+"){
		  boubou.CSM=boubou.CSMP ;
		  boubou.DT=boubou.DTION ;
		  }
      if (particule0=="ion He++"){
		  boubou.CSM=boubou.CSMA ;
		  boubou.DT=boubou.DTION ;
		  }
      if (particule0=="ion O+"){
		  boubou.CSM=boubou.CSMO ;
		  boubou.DT=boubou.DTION ;
		  }

      boubou.AK0=energie0 ;
      boubou.ALPHA=alpha0 ;
      boubou.BETA=beta0 ;
      boubou.XER=xer0 ;
      boubou.YER=yer0 ;
      boubou.ZER=zer0 ;

//      if (energie0==10000) throw new SimuException("Electron non relativiste");

      test_initialise=boubou.Initialise();
      System.out.println("-Projectile- test_initialise : "+test_initialise);
      System.out.println("-Projectile- alpha0 = "+alpha0);
      System.out.println("-Projectile- ALPHA  = "+boubou.ALPHA);
      System.out.println("-Projectile- CSM    = "+boubou.CSM);
      System.out.println("-Projectile- CSMI   = "+boubou.CSMI);
      System.out.println("-Projectile- energie0 = "+energie0);
      System.out.println("-Projectile- AK0      = "+boubou.AK0);
      System.out.println("-Projectile- VXE      = "+boubou.VXE);
      System.out.println("-Projectile- VYE      = "+boubou.VYE);
      System.out.println("-Projectile- VV       = "+boubou.VV);
      System.out.println("-Projectile- COSB     = "+boubou.COSB);
      System.out.println("-Projectile- BERAD    = "+boubou.BERAD);
      System.out.println("-Projectile- DT       = "+boubou.DT);
      System.out.println("-Projectile- fin de initCalculLive");
    }

 // ///////////////// CALCUL LIVE /////////////////////////////////////

    public ListeValeurs calculLive() throws SimuException {
        if (x < boubou.Xrange0p |x > boubou.Xrange1p
           |y < boubou.Yrange0p |y > boubou.Yrange1p)
//           | istep > boubou.NSTEP)
           {System.out.println("-Projectile- fin du calcul : "+test_increment);
            System.out.println("-Projectile- x= "+x+"  y="+y+"  z="+z);
			 return null; }// fin du calcul
        ListeValeurs out = new ListeValeurs();
        test_increment = boubou.Increment();
//        System.out.println("-Projectile- test_increment : "+test_increment);
//        System.out.println("-Projectile- test_increment : "+test_increment);

        istep=istep+1;
        x = boubou.XER;
        y = boubou.YER;
        z = boubou.ZER;
        vx= boubou.VXE;
        vy= boubou.VYE;
        vz= boubou.VZE;


        out.ajouterDouble("x",x);
        out.ajouterDouble("y",y);
        out.ajouterDouble("z",z);


/*
        try { // pour bloquer le programme pendant dt secondes
            Thread.currentThread().sleep(Math.round(dt*20));
        } catch (InterruptedException e) {}
*/

        return out;
    }


 // ///////////////// CALCUL COMPLET /////////////////////////////////////

// QUESTION pour DAMIEN :
// ici il y a un probleme de cadrage.
// les contours du cadre ne sont definis que pour Plot Live.
// quoi faire avec Plot ?

    // calcul complet d'un coup (pas utilis avec affichage d'un plot live)
    public ListeValeurs calcul(Vector in) throws SimuException {
/*        x = 0;
        y = 0;
        double energie0 = getDouble(in, 0);
        double alpha0   = getDouble(in, 1);
        double beta0    = getDouble(in, 2);
//        vx = v0 * Math.cos(angle/180*Math.PI);
//        vy = v0 * Math.sin(angle/180*Math.PI);
      boubou.AK0=energie0 ;
      boubou.ALPHA=alpha0 ;
      boubou.BETA=beta0 ;
      test_initialise=boubou.Initialise();
      System.out.println("-Projectile- test_initialise : "+test_initialise);

        Vector out = new Vector();
        Vector outx = new Vector();
        Vector outy = new Vector();

        istep=0;

        while (  x > boubou.Xrange0p &&x < boubou.Xrange1p
              && y > boubou.Yrange0p &&y < boubou.Yrange1p
              && istep < boubou.NSTEP) {


        test_increment = boubou.Increment();
		System.out.println("-Projectile- test_increment : "+test_increment);
		System.out.println("-Projectile- istep          : "+istep);
		System.out.println("-Projectile- boubou.NSTEP : "+boubou.NSTEP);

		istep++; //incremente le pas de temps
		x = boubou.XER;
		y = boubou.YER;
		vx= boubou.VXE;
        vy= boubou.VYE;

        outx.addElement(new Double(x));
        outy.addElement(new Double(y));
        } // fin while


        out.addElement(outx);
        out.addElement(outy);
*/
        return null;
    }
}
